/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area;

import net.wordrider.area.actions.RedoAction;
import net.wordrider.area.actions.UndoAction;

import javax.swing.event.UndoableEditEvent;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;
import javax.swing.undo.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * @author Vity
 */
final class AreaUndoManager extends UndoManager implements PropertyChangeListener {

    private final RiderArea editor;
    private CompoundEdit compoundEdit;
    private int lastOffset;
    private boolean multipleUndo = false;
    private boolean multipleUndoFirstTime = false;


    public AreaUndoManager(final RiderArea editor) {
        this.editor = editor;
        editor.getDocument().addUndoableEditListener(this);
        editor.addPropertyChangeListener("document", this);
        editor.addPropertyChangeListener("undoredo", this);
    }

    /*
    **  Listen for Undoable Edits
    */
//    public final void xxxundoableEditHappened(final UndoableEditEvent e) {
//        addEdit(e.getEdit());
//        getUndoAction().updateUndoState();
//        getRedoAction().updateRedoState();
//    }

    public final boolean addEdit(final UndoableEdit edit) {
        final boolean result = super.addEdit(edit);

        UndoAction.getInstance().updateUndoState(editor);
        editor.setModified(this.canUndoOrRedo());
        RedoAction.getInstance().updateRedoState(editor);
        return result;
    }


    /*
**  Whenever an UndoableEdit happens the edit will either be absorbed
**  by the current compound edit or a new compound edit will be started
*/
    public final void undoableEditHappened(final UndoableEditEvent e) {
        // Start a new compound edit
        if (multipleUndoFirstTime) {
            if (compoundEdit != null) compoundEdit.end();
            compoundEdit = startCompoundEdit(e.getEdit());
            multipleUndoFirstTime = false;
            return;
        } else if (compoundEdit == null) {
            compoundEdit = startCompoundEdit(e.getEdit());
            return;
        }
        //  Check for an incremental edit or backspace

        final int diff = editor.getCaretPosition() - lastOffset;
        if (Math.abs(diff) == 1 || multipleUndo) {
            compoundEdit.addEdit(e.getEdit());
            editor.setModified(this.canUndoOrRedo());
            lastOffset += diff;
            return;
        }

        //  Not incremental edit, end previous edit and start a new one

        compoundEdit.end();
        compoundEdit = startCompoundEdit(e.getEdit());
    }

    /*
    **  Each CompoundEdit will store a group of related incremental edit
    **  (ie. each character typed or backspaced is an incremental edit)
    */
    private final CompoundEdit startCompoundEdit(final UndoableEdit anEdit) {
        //  Track the starting offset of this compound edit

        lastOffset = editor.getCaretPosition();

        //  The compound edit is used to store incremental edits

        compoundEdit = new MyCompoundEdit(lastOffset);
        compoundEdit.addEdit(anEdit);

        //  The compound edit is added to the UndoManager. All incremental
        //  edits stored in the compound edit will be undone/redone at once

        addEdit(compoundEdit);
        return compoundEdit;
    }

    public synchronized void discardAllEdits() {
        super.discardAllEdits();    //call to super
        UndoAction.getInstance().updateUndoState(editor);
        RedoAction.getInstance().updateRedoState(editor);
    }

    private final class MyCompoundEdit extends CompoundEdit {
        private int caretPosition;

        public MyCompoundEdit(final int caretPosition) {
            super();    //call to super
            this.caretPosition = caretPosition;
        }

        public final boolean isInProgress() {
            //  in order for the canUndo() and canRedo() methods to work
            //  assume that the compound edit is never in progress

            return false;
        }

        public final void undo() throws CannotUndoException {
            //  End the edit so future edits don't get absorbed by this edit

            if (compoundEdit != null)
                compoundEdit.end();

            super.undo();
            updateEditor();
            //  Always start a new compound edit after an undo

            compoundEdit = null;
        }

        private void updateEditor() {
            if (caretPosition < editor.getText().length()) {
                editor.setCaretPosition(caretPosition);
            }
            editor.validate();
            editor.repaint();
        }

        public void redo() throws CannotRedoException {
            super.redo();    //call to super
            updateEditor();
        }

    }


    /**
     * This method gets called when a bound property is changed.
     *
     * @param event A PropertyChangeEvent object describing the event source
     *              and the property that has changed.
     */

    public final void propertyChange(final PropertyChangeEvent event) {
        if (event.getSource() == getEditor()) {
            if (event.getPropertyName().equals("document")) {

                // Reset the UndoableEditListener
                ((Document) event.getOldValue()).removeUndoableEditListener(this);
                final Document newDoc = (Document) event.getNewValue();
                newDoc.addUndoableEditListener(this);
            } else if (event.getPropertyName().equals("undoredo")) {
                this.multipleUndo = this.multipleUndoFirstTime = ((Boolean) event.getNewValue()).booleanValue();
            }
        }
    }


    private final JTextComponent getEditor() {
        return editor;
    }

}
