/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.area;

//import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.text.*;
import java.awt.*;
import java.util.Collection;
import java.util.LinkedList;

//import java.awt.image.BufferedImage;

/**
 * @author Vity
 */
public final class RiderDocument extends DefaultStyledDocument {
    private static final int CHARACTERS_PER_TAB = 2;

    /**
     * EOL tag that we re-use when creating ElementSpecs
     */
    private static final char[] EOL_ARRAY = {'\n'};

    /**
     * Batched ElementSpecs
     */
    private Collection batch = null;
    private static final Element[] ELEMENT = new Element[0];

    public RiderDocument() {
        super();
        RiderStyles.createDefaultStyles((StyleContext) getAttributeContext());
        batch = new LinkedList();
        this.putProperty(PlainDocument.tabSizeAttribute, new Integer(CHARACTERS_PER_TAB));
    }

    public void appendBatchString(final char[] str,
                                  final AttributeSet a) {
        // We could synchronize this if multiple threads
        // would be in here. Since we're trying to boost speed,
        // we'll leave it off for now.

        // Make a copy of the attributes, since we will hang onto
        // them indefinitely and the caller might change them
        // before they are processed.
        batch.add(new ElementSpec(a.copyAttributes(), ElementSpec.ContentType, str, 0, str.length));
    }

    /**
     * Adds a String (assumed to not contain linefeeds) for
     * later batch insertion.
     */
    public void appendBatchString(final String str,
                                  final AttributeSet a) {
        // We could synchronize this if multiple threads
        // would be in here. Since we're trying to boost speed,
        // we'll leave it off for now.

        // Make a copy of the attributes, since we will hang onto
        // them indefinitely and the caller might change them
        // before they are processed.
        batch.add(new ElementSpec(a.copyAttributes(), ElementSpec.ContentType, str.toCharArray(), 0, str.length()));
    }

    /**
     * Adds a linefeed for later batch processing
     */
    public void appendBatchLineFeed(final AttributeSet a) {
        // See sync notes above. In the interest of speed, this
        // isn't synchronized.

        // Add a spec with the linefeed characters
        batch.add(new ElementSpec(a, ElementSpec.ContentType, EOL_ARRAY, 0, 1));

        // Then add attributes for element start/end tags. Ideally
        // we'd get the attributes for the current position, but we
        // don't know what those are yet if we have unprocessed
        // batch inserts. Alternatives would be to get the last
        // paragraph element (instead of the first), or to process
        // any batch changes when a linefeed is inserted.
//        Element paragraph = ;
//        AttributeSet pattr = paragraph.getAttributes();
        batch.add(new ElementSpec(null, ElementSpec.EndTagType));
        batch.add(new ElementSpec(getParagraphElement(0).getAttributes(), ElementSpec.StartTagType));
    }
  

    public void processBatchUpdates(final int offs) throws
            BadLocationException {
        // As with insertBatchString, this could be synchronized if
        // there was a chance multiple threads would be in here.
        final ElementSpec[] inserts = new ElementSpec[batch.size()];
        batch.toArray(inserts);
        batch = new LinkedList();
        // Process all of the inserts in bulk
        super.insert(offs, inserts);
    }

    public final Font getFont(final AttributeSet attr) {
        // PENDING(prinz) add cache behavior
        int style = Font.PLAIN;
        if (StyleConstants.isBold(attr))
            style |= Font.BOLD;
        if (StyleConstants.isItalic(attr))
            style |= Font.ITALIC;
        final String family = RiderStyles.FONT_FAMILY;
        final Integer aSize = (Integer) attr.getAttribute(StyleConstants.FontSize);
        int size = (aSize != null) ? aSize.intValue() : RiderStyles.SIZE_NORMAL;

        /**
         * if either superscript or subscript is
         * is set, we need to reduce the font size
         * by 2.
         */
        if (StyleConstants.isSuperscript(attr) ||
                StyleConstants.isSubscript(attr)) {
            size -= 1;
        }
        return ((StyleContext) getAttributeContext()).getFont(family, style, size);
    }

    public final void toggleBookmark(final int offset) {
        toggleBookmark(getParagraphElement(offset));
    }


    public final void putBookmark(final int offset) {
        setParagraphAttributes(offset,
                0, RiderStyles.updateBookmark(getParagraphElement(offset)), false);
    }

    public void removeBookmark(final Element el) {
        final AttributeSet set = getAttributeContext().removeAttributes(el.getAttributes(), RiderStyles.bookmarkStyle.getAttributeNames());
        setParagraphAttributes(el.getStartOffset(),
                0, set, true);
    }

    public final void toggleBookmark(final Element el) {
        if (RiderStyles.isMath(el) || RiderStyles.isReadonlySection(el))
            return;
        final int elLength = el.getEndOffset() - el.getStartOffset();
        if (RiderStyles.isBookmark(el)) {
            removeBookmark(el);
        } else
            setParagraphAttributes(el.getStartOffset(),
                    elLength, RiderStyles.updateBookmark(el), false);
        refresh(el.getStartOffset(), elLength);
    }

    public void activatePrettyPrint(int pos) {
        final Element el = getParagraphElement(pos);
        final int elLength = el.getEndOffset() - el.getStartOffset();
        removeBookmark(el);
        setCharacterAttributes(el.getStartOffset(), elLength, new SimpleAttributeSet(), true);
        setParagraphAttributes(el.getStartOffset(),
                elLength, RiderStyles.mathStyle, true);
    }

    public void setPrettyPrint(final Element el) {
        final int elLength = el.getEndOffset() - el.getStartOffset();
        if (RiderStyles.isMath(el)) {
            final AttributeSet set = getAttributeContext().removeAttributes(el.getAttributes(), RiderStyles.mathStyle.getAttributeNames());
            setParagraphAttributes(el.getStartOffset(),
                    elLength, set, true);
        } else {
            removeBookmark(el);

            setCharacterAttributes(el.getStartOffset(), elLength, new SimpleAttributeSet(), true);
            setParagraphAttributes(el.getStartOffset(),
                    elLength, RiderStyles.mathStyle, true);
        }
        refresh(el.getStartOffset(), elLength);
    }

    public final void refresh(final int offset, final int len) {
//        if (offset == 0)
//            len = getLength();
        final DefaultDocumentEvent changes = new DefaultDocumentEvent(offset, len, DocumentEvent.EventType.CHANGE);
        final Element root = getDefaultRootElement();
        final Element[] removed = ELEMENT;
        final Element[] added = ELEMENT;
        changes.addEdit(new ElementEdit(root, 0, removed, added));
        changes.end();
        fireChangedUpdate(changes);
    }

    public final void insertPicture(final int offset, final Image image) throws BadLocationException {
//        final BufferedImage bi = new BufferedImage(image.getWidth(null),
//                image.getHeight(null),
//                BufferedImage.TYPE_BYTE_BINARY);
//        bi.createGraphics().drawImage(image, 0, 0, null);

        final MutableAttributeSet attr = new SimpleAttributeSet();
        StyleConstants.setIcon(attr, new ImageIcon(image));
        //inserts image in the document
        this.insertString(offset, " ", attr);
    }

    public final void insertSeparateLine(final JTextComponent pane, final int offset, final int lineType) throws BadLocationException {
        //AttributeSet set = getParagraphElement(offset).getAttributes();
        setParagraphAttributes(offset, 0, RiderStyles.aligmentLeftStyle, true);
        final MutableAttributeSet attr = new SimpleAttributeSet();
        StyleConstants.setComponent(attr, new SeparatorLine(pane, lineType));
        this.insertString(offset, " ", attr);
        //this.insertString(offset, "\nasd", null);
        //setParagraphAttributes(offset, 0, set, true);
    }

}
