/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.actions;

import net.wordrider.area.RiderArea;
import net.wordrider.core.AppPrefs;
import net.wordrider.core.managers.AreaManager;
import net.wordrider.core.managers.FileInstance;
import net.wordrider.ti89.InvalidDataTypeException;
import net.wordrider.ti89.NotSupportedFileException;
import net.wordrider.ti89.TITextFileReader;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.IOException;

/**
 * @author Vity
 */
public final class OpenFileAction extends CoreAction {

    private static final OpenFileAction instance = new OpenFileAction();
    public static final String LASTOPENFOLDER_KEY = "lastOpenFolder";
    private static final String CODE = "OpenFileAction";


    public static OpenFileAction getInstance() {
        return instance;
    }

    private OpenFileAction() {
        super(CODE, KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK), "open.gif");
    }


    public final static class File89tFilter extends javax.swing.filechooser.FileFilter implements IFileType {
        private static final String FILE_EXTENSION = "89t";

        public final String getDescription() {
            return AppPrefs.getLabel(CODE + ".dialog.filter89");  //implement - call to super class
        }

        public String getExtension() {
            return FILE_EXTENSION;  //implement - call to super class
        }

        //Accept all directories and all gif, jpg, tiff, or png files.
        public final boolean accept(final File f) {
            if (f.isDirectory())
                return true;
            final String extension = Utils.getExtension(f);
            if (extension != null) {
                if (extension.equals(FILE_EXTENSION))
                    return true;
                else
                    return false;
            }
            return false;
        }

    }

    private File getOpenFileDialog() {
        final File currentDirectory = new File(AppPrefs.getProperty(LASTOPENFOLDER_KEY, ""));
        //final JFileChooser fileDialog = (currentDirectory != null && currentDirectory.exists()) ? new JFileChooser(currentDirectory) : new JFileChooser();
        final JFileChooser fileDialog = new JFileChooser(currentDirectory);
        fileDialog.setDialogTitle(AppPrefs.getLabel(CODE + ".dialog.title"));
        fileDialog.setFileFilter(new File89tFilter());
        fileDialog.showOpenDialog(getMainFrame());
        return fileDialog.getSelectedFile();
    }

    public final boolean open(final File openFile) {
        final File f = (openFile == null) ? getOpenFileDialog() : openFile;
        if (f == null)
            return false;
        if (!f.isFile() || !f.exists()) {
            Swinger.showErrorDialog(getMainFrame(), AppPrefs.getLabel("message.error.FNF", f.getAbsolutePath()));
            return false;
        }
        AppPrefs.storeProperty(LASTOPENFOLDER_KEY, f.getAbsolutePath());
        final AreaManager areaManager = getAreaManager();
        FileInstance instance = (FileInstance) areaManager.isFileAlreadyOpened(f);
        final boolean reload;
        if (instance != null) {
            if (Swinger.getChoice(getMainFrame(), AppPrefs.getLabel("message.confirm.FAE")) == Swinger.RESULT_YES)
                reload = true;
            else
                return false;
        } else
            reload = false;

        try {
            final TITextFileReader file = new TITextFileReader();
            file.openFromFile(f);
            if (!reload) {
                instance = new FileInstance(f, file.getTextFileInfo());
                areaManager.openFileInstance(instance);
            }
            final RiderArea area = (RiderArea) instance.getTextComponent();
            area.noModifyUpdate(true);
            area.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            try {
                final Document doc = area.getDocument();
                final int length = doc.getLength();
                if (length > 0)
                    doc.remove(0, length);
                doc.insertString(0, AppPrefs.getLabel("message.loading"), null);
            } catch (BadLocationException ex) {
                Utils.processException(ex);
            }
            area.getUndoManager().discardAllEdits();
            area.noModifyUpdate(false);
            // frame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            if (reload)
                areaManager.setActivateFileInstance(instance);
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    try {
                        area.getKit().setTextRiderText(area, file.getContent(), f.getParent());
                        // net.wordrider.area.setModified(false);
                    } catch (BadLocationException ex) {
                        Utils.processException(ex);
                        Swinger.showErrorDialog(getMainFrame(), ex.getMessage());
                        return;
                    }
                }
            });
        } catch (InvalidDataTypeException ex) {
            Utils.log(Utils.WARN_LOG, ex.getMessage());
            Swinger.showErrorDialog(getMainFrame(), ex.getMessage());
            return false;
        } catch (NotSupportedFileException ex) {
            Utils.log(Utils.WARN_LOG, ex.getMessage());
            Swinger.showErrorDialog(getMainFrame(), ex.getMessage());
            return false;
        } catch (IOException ex) {
            Utils.processException(ex);
            Swinger.showErrorDialog(getMainFrame(), AppPrefs.getLabel("message.error.reading", ex.getMessage()));
            return false;
        } finally {
            areaManager.getActiveFileInstance().getTextComponent().setCursor(Cursor.getDefaultCursor());
        }//catch (BadLocationException ex) {
//            Utils.processException(ex);
//            return;
//        }
        return true;
    }

    public final void actionPerformed(final ActionEvent e) {
        open(null);
    }


}