/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.actions;

import net.wordrider.area.RiderStyles;
import net.wordrider.ti89.TIImageDecoder;
import net.wordrider.ti89.TITextFileInfo;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.StyledDocument;

/**
 * @author Vity
 */

/**
 * #                          #                    &
 * U : underline              1 : small font       L : Left alignement
 * V : vector                 2 : medium font      R : Right alignement
 * I : invert                 3 : big font         C : Centered
 * N : underline dotted                            \ : Margin 0 pixel
 * S : strikethrought                              , : Margin 10 pixels
 * E : exposant                                    ; : Margin 20 pixels
 * W : Word Wrap                                   . : margin 30 pixels
 * - : line separator                              E : Pretty printed expression
 * = : double "-"
 */
final class TextRiderProcess {


    private static final String[] txtriderTags = {"#U", "#V", "#I", "#N", "#S", "#E", "#W"};
    private static final String[] txtriderParagraphTags = {"#1", "#2", "#3", "&L", "&R", "&C", "&\\", "&,", "&;", "&.", "&-", "&=", "&E", "&P"};
    private final boolean[] paarAttributes = new boolean[txtriderTags.length];
    private final short[] attributesMore = new short[3];

    private static final byte P_ALIGNMENT = 0;
    private static final byte P_SIZE = 1;
    private static final byte P_MARGIN = 2;

    private static final byte P_UNDERLINE = 0;
    private static final byte P_VECTOR = 1;
    private static final byte P_INVERT = 2;
    private static final byte P_UNDERLINE_DOTTED = 3;
    private static final byte P_STRIKETHROUGHT = 4;
    private static final byte P_EXPOSANT = 5;
    private static final byte P_WORDWRAP = 6;
    private static final byte P_SIZEMINI = 0;
    private static final byte P_SIZENORMAL = 1;
    private static final byte P_SIZEMAXI = 2;
    private static final byte P_ALIGNLEFT = 3;
    private static final byte P_ALIGNRIGHT = 4;
    private static final byte P_ALIGNCENTER = 5;
    private static final byte P_MARGIN0 = 6;
    private static final byte P_MARGIN10 = 7;
    private static final byte P_MARGIN20 = 8;
    private static final byte P_MARGIN30 = 9;
    private static final byte P_LINESEPARATOR = 10;
    private static final byte P_DOUBLELINE = 11;
    private static final byte P_PRETTYPRINT = 12;
    private static final byte P_PICTURE = 13;

    private static final String LINE_SEPARATOR_TEXT = "\n";
    private static final String LINE_SEPARATOR_FILE = "\r ";
    private static final String tabulator = "   ";
    private String activeLineSeparator = LINE_SEPARATOR_TEXT;
    private StringBuffer buffer;
    private boolean prettyPrint = false;
    private AttributeSet attr;
    private TITextFileInfo info;


    private void initProcess() {
        for (int i = 0; i < paarAttributes.length; ++i)
            paarAttributes[i] = false;
        attributesMore[P_ALIGNMENT] = P_ALIGNLEFT;
        attributesMore[P_SIZE] = P_SIZENORMAL;
        attributesMore[P_MARGIN] = P_MARGIN0;
    }


//    public final String getTxtRiderTextForSelection(final StyledDocument document, final int startElemenIndex, final int endElemenIndex, final int startOffset, final int endOffset) throws BadLocationException {
//        final Element sectionElem = document.getDefaultRootElement();
//        activeLineSeparator = LINE_SEPARATOR_FILE;
//        buffer = new StringBuffer(endOffset - startOffset + 50);
//        initProcess();
//        int elCount;
//        String text;
//        Element element;
//        boolean wasSetComponent;
//        for (int i = startElemenIndex; i <= endElemenIndex; i++) {
//            element = sectionElem.getElement(i);
//            updateBookmark(element);
//            updateParagraphProperties(element);
//            elCount = element.getElementCount();
//            wasSetComponent = false;
//            for (int j = 0; j < elCount; j++) {
//                element = element.getElement(j);
//                if (updateComponents(element, j == 0))
//                    wasSetComponent = true;
//                else {
//                    final int rangeStart = element.getStartOffset();
//                    final int rangeEnd = element.getEndOffset();
//                    text = document.getText(rangeStart, rangeEnd - rangeStart);
//                    if (!text.equals("\n") && !text.equals("\n\r") && wasSetComponent)
//                        buffer.append(activeLineSeparator);
//                    if (!prettyPrint)
//                        updateCharacterAttributes(element.getAttributes());
//                    //after adding attributes!
//                    buffer.append(text);
//                }
//            }
//        }
//        return updateResult(true);
//    }


    public final String getTxtRiderTextForDocument(final StyledDocument document, final TITextFileInfo info, final boolean fileOutput) throws BadLocationException {
        final Element sectionElem = document.getDefaultRootElement();
        final int paraCount = sectionElem.getElementCount();
        activeLineSeparator = (fileOutput) ? LINE_SEPARATOR_FILE : LINE_SEPARATOR_TEXT;
        buffer = new StringBuffer(document.getLength() + 50);
        this.info = info;
        initProcess();
        int elCount;
        String text;
        Element element, paraEl;
        boolean wasSetComponent;
        for (int i = 0; i < paraCount; i++) {
            paraEl = sectionElem.getElement(i);
            if (fileOutput)
                updateBookmark(paraEl);
            if (!RiderStyles.isReadonlySection(paraEl))
                updateParagraphProperties(paraEl);
            elCount = paraEl.getElementCount();
            wasSetComponent = false;
            for (int j = 0; j < elCount; j++) {
                element = paraEl.getElement(j);
                if (updateComponents(element, j == 0))
                    wasSetComponent = true;
                else {
                    final int rangeStart = element.getStartOffset();
                    final int rangeEnd = element.getEndOffset();
                    text = document.getText(rangeStart, rangeEnd - rangeStart);
                    if (!text.equals("\n") && !text.equals("\n\r") && wasSetComponent)
                        buffer.append(activeLineSeparator);
                    if (!prettyPrint && !wasSetComponent)
                        updateCharacterAttributes(element.getAttributes());
                    //after adding attributes!
                    buffer.append(text);
                }
            }
        }
        return updateResult(fileOutput);
    }

    private boolean updateComponents(final Element element, final boolean isFirstInOrder) {
        return updateSingleLine(element, isFirstInOrder) || updateDoubleLine(element, isFirstInOrder) || updateImageComponent(element, isFirstInOrder);
    }

    private void updateParagraphProperties(final Element element) {
        if (!(prettyPrint = updatePrettyPrint(element))) {
            attr = element.getAttributes();
            updateMargin(attr);
            updateAligment(attr);
        }
    }


    private void updateCharacterAttributes(final AttributeSet attr) {
        updatePaarTag(P_EXPOSANT, RiderStyles.isExposant(attr)); //must be before updateFontSize
        updateFontSize(attr);
        updatePaarTag(P_WORDWRAP, RiderStyles.isWordWrap(attr));
        updatePaarTag(P_UNDERLINE, RiderStyles.isUnderLine(attr));
        updatePaarTag(P_INVERT, RiderStyles.isInvert(attr));
        updatePaarTag(P_UNDERLINE_DOTTED, RiderStyles.isUnderLineDotted(attr));
        updatePaarTag(P_STRIKETHROUGHT, RiderStyles.isStrikeOut(attr));
        updatePaarTag(P_VECTOR, RiderStyles.isVector(attr));
    }


    private String updateResult(final boolean fileOutput) {
        final String result;
        if (fileOutput) {
            buffer.deleteCharAt(buffer.length() - 1); //the last line separator delete
            if (buffer.charAt(buffer.length() - 1) == '\n') buffer.append(' ');//space after the last \r if any
            result = buffer.toString().replaceAll("\n\r", "\r ").replaceAll("\t", tabulator);
            buffer = null;
            return result;
        }
        result = buffer.toString();
        buffer = null;//free memory
        return result;
    }

    private void updateBookmark(final Element el) {
        buffer.append((RiderStyles.isBookmark(el)) ? '\f' : ' ');
    }

    private boolean updatePrettyPrint(final Element el) {
        final boolean isMath = RiderStyles.isMath(el);
        if (isMath)
            buffer.append(txtriderParagraphTags[P_PRETTYPRINT]);
        return isMath;
    }

    private final void updateAligment(final AttributeSet attributeSet) {
        final short align;
        switch (RiderStyles.getAligment(attributeSet)) {
            case RiderStyles.ALIGN_LEFT:
                align = P_ALIGNLEFT;
                break;
            case RiderStyles.ALIGN_CENTER:
                align = P_ALIGNCENTER;
                break;
            case RiderStyles.ALIGN_RIGHT:
                align = P_ALIGNRIGHT;
                break;
            default:
                align = P_ALIGNLEFT;
                break;
        }
        if (attributesMore[P_ALIGNMENT] != align) {
            attributesMore[P_ALIGNMENT] = align;
//            if (buffer.length() != 1) //space at the beginning
//                buffer.append(activeLineSeparator);
            buffer.append(txtriderParagraphTags[align]);
        }
    }

    private final void updateMargin(final AttributeSet attributeSet) {
        final short margin;
        switch (RiderStyles.getMargin(attributeSet)) {
            case RiderStyles.MARGIN_0:
                margin = P_MARGIN0;
                break;
            case RiderStyles.MARGIN_10:
                margin = P_MARGIN10;
                break;
            case RiderStyles.MARGIN_20:
                margin = P_MARGIN20;
                break;
            case RiderStyles.MARGIN_30:
                margin = P_MARGIN30;
                break;
            default:
                margin = P_MARGIN0;
                break;
        }
        if (attributesMore[P_MARGIN] != margin) {
            attributesMore[P_MARGIN] = margin;
            //if (buffer.length() != 1) //bookmark at the beginning = 1
            //    buffer.append(activeLineSeparator);
            buffer.append(txtriderParagraphTags[margin]);
        }
    }

    private final void updateFontSize(final AttributeSet attributeSet) {
        final short size;
        switch (RiderStyles.getFontSize(attributeSet)) {
            case RiderStyles.SIZE_NORMAL:
                size = P_SIZENORMAL;
                break;
            case RiderStyles.SIZE_MAXI:
                size = P_SIZEMAXI;
                break;
            case RiderStyles.SIZE_MINI:
                size = P_SIZEMINI;
                break;
            default:
                size = P_SIZENORMAL;
                break;
        }
        if (attributesMore[P_SIZE] != size) {
            attributesMore[P_SIZE] = size;
            buffer.append(txtriderParagraphTags[size]);
        }
    }

    private final boolean updateSingleLine(final Element el, final boolean isFirstInOrder) {
        if (RiderStyles.isComponentLine(el, RiderStyles.SINGLE_LINE)) {
            if (!isFirstInOrder)
                buffer.append(activeLineSeparator);
            buffer.append(txtriderParagraphTags[P_LINESEPARATOR]);
            return true;
        }
        return false;
    }

    private final boolean updateDoubleLine(final Element el, final boolean isFirstInOrder) {
        if (RiderStyles.isComponentLine(el, RiderStyles.DOUBLE_LINE)) {
            if (!isFirstInOrder)
                buffer.append(activeLineSeparator);
            buffer.append(txtriderParagraphTags[P_DOUBLELINE]);
            return true;
        }
        return false;
    }

    private final String generateImageDesc(final TIImageDecoder tiImage) {
        switch (info.getPictureProcessingType()) {
            case TITextFileInfo.PICTURE_FOLDER_DONTCHANGE:
                return tiImage.getFileInfo().getFolderName() + "\\" + tiImage.getFileInfo().getVarName();
            case TITextFileInfo.PICTURE_FOLDER_USELAST:
                final int lastIndex = tiImage.getImagePath().lastIndexOf('\\') + 1;
                if (lastIndex != 0) {
                    return tiImage.getImagePath().substring(lastIndex) + "\\" + tiImage.getFileInfo().getVarName();
                } else
                    return null;
            case TITextFileInfo.PICTURE_FOLDER_USEOWN:
                return info.getPictureFolder() + "\\" + tiImage.getFileInfo().getVarName();
            case TITextFileInfo.PICTURE_USESAMEASFORDOCUMENT:
                return info.getFolderName() + "\\" + tiImage.getFileInfo().getVarName();
            default :
                return null;
        }
    }

    private final boolean updateImageComponent(final Element el, final boolean isFirstInOrder) {
        final TIImageDecoder tiImage;
        if ((tiImage = RiderStyles.isSupportedImage(el)) != null) {
            final String imageDesc;
            if ((imageDesc = generateImageDesc(tiImage)) != null) {
                if (!isFirstInOrder)
                    buffer.append(activeLineSeparator);
                buffer.append(txtriderParagraphTags[P_PICTURE]).append(imageDesc);
                return true;
            }
        }
        return false;
    }


    private final void updatePaarTag(final int tag, final boolean isSet) {
        if (isSet ^ paarAttributes[tag]) {
            paarAttributes[tag] = isSet;
            buffer.append(txtriderTags[tag]);
        }
    }


}
