/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.core.managers;

import net.wordrider.core.AppPrefs;
import net.wordrider.core.MainApp;
import net.wordrider.core.managers.interfaces.IAreaChangeListener;
import net.wordrider.core.managers.interfaces.IFileInstance;
import net.wordrider.core.managers.interfaces.IHideAble;
import net.wordrider.core.managers.interfaces.IRiderManager;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.JTextComponent;
import javax.swing.text.Utilities;
import java.awt.*;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * @author Vity
 */
public final class StatusbarManager implements IRiderManager, CaretListener, IAreaChangeListener, PropertyChangeListener, IHideAble {
    public static final String MODIFIED_PROPERTY = "modified";

    private final JPanel statusPanel;
    private static final String LINE_PROPERTY = "linePosition";

    private static final String OVERTYPE_PROPERTY = "overtype";

    private JTextComponent editor;
    private final Map subSections = new LinkedHashMap(9);
    private static final Font font = new Font("SansSerif", Font.PLAIN, 11);
    //  private final Box box = new Box(BoxLayout.Y_AXIS);

    public StatusbarManager() {
        statusPanel = new JPanel();
        statusPanel.setLayout(new BoxLayout(statusPanel, BoxLayout.X_AXIS));
        statusPanel.setSize(new Dimension(10, 16));
        statusPanel.setPreferredSize(new Dimension(10, 16));
        statusPanel.setVisible(AppPrefs.getProperty("net.wordrider.gui.showStatusbar", true));
        init();
    }

    private void init() {
        clear();
    }

    private void addSeparator(final String sectionName, final int length) {
        removeSection(sectionName);
        subSections.put(sectionName, new PanelItem("", null, length, true));
        repaintStatusPanel();
    }

    private void addSection(final String sectionName, final String staticLabel, final String hint, final int length) {
        removeSection(sectionName);
        subSections.put(sectionName, new PanelItem(staticLabel, hint, length, false));
        repaintStatusPanel();
    }

    private void removeSection(final String sectionName) {
        if (subSections.containsKey(sectionName)) {
            subSections.remove(sectionName);
            repaintStatusPanel();
        }
    }

    private void display(final String sectionName, final String value) {
        //if (subSections.containsKey(sectionName)) {
        final PanelItem item = (PanelItem) subSections.get(sectionName);
        item.setOptionalLabel(value);
        //}
    }

    private void display(final String sectionName, final String value, final String hint) {
        //if (subSections.containsKey(sectionName)) {
        final PanelItem item = (PanelItem) subSections.get(sectionName);
        item.setOptionalLabel(value, hint);
        //}
    }

    private void repaintStatusPanel() {
        // clear all panel
        clear();
        // creates default GridBagConstraints for all..
        // read one by one
        PanelItem item = null;
        //int pos = 0;
        for (final Iterator it = subSections.values().iterator(); it.hasNext();) {
            item = (PanelItem) it.next();
            final Dimension dimension = new Dimension(item.getLength(), 14);
            // for any creates panel and add there
//            if (pos > 1)
//                constraints.anchor = GridBagConstraints.EAST;
            final JPanel cell = new JPanel(null);
            cell.add(item.getComponent());
            //  cell.setBorder(BorderFactory.createEtchedBorder());
            cell.setPreferredSize(dimension);
            if (!item.isSeparator())
                cell.setMinimumSize(dimension);
            //this.statusPanel.add(cell, constraints);
            this.statusPanel.add(cell);
//            this.statusPanel.add(Box.createVerticalBox());
//            this.statusPanel.add(Box.createVerticalGlue());
//            this.statusPanel.add(Box.createVerticalStrut(10));
        }
        this.statusPanel.validate();
    }


    private void clear() {
        if (statusPanel != null) {
            statusPanel.removeAll();
            //addStaticPanel();
            this.statusPanel.validate();
            this.statusPanel.repaint();
        }
    }

//    private void addStaticPanel() {
//        final GridBagConstraints constraints = new GridBagConstraints(50, 0, 1, 1, 1.0D, 0, GridBagConstraints.EAST, GridBagConstraints.HORIZONTAL, new Insets(1, 0, 1, 0), 0, 0);
//        final JPanel staticPanel = new JPanel();
//        //staticPanel.setBorder(new SoftBevelBorder(SoftBevelBorder.LOWERED));
//        staticPanel.setPreferredSize(new java.awt.Dimension(0, 14));
    //     statusPanel.add(staticPanel, constraints);

//    }

    public void displayFilePath(final String newValue) {
        display("fileName", Utils.shortenFileName(newValue, 50), newValue);
    }

    private void displayOvertype(final Boolean value) {
        display(OVERTYPE_PROPERTY, ((value != null && value.booleanValue())) ? AppPrefs.getLabel("statusbar.rewrite") : AppPrefs.getLabel("statusbar.insert"));
    }

    private final class PanelItem {
        private String staticLabel = "";
        private String optionalLabel = "";
        private int length;
        private Component component;
        private final static int COMPONENT_TYPE_LABEL = 0;
        private final int componentType = COMPONENT_TYPE_LABEL;
        private boolean separator = false;


        private PanelItem() {
            super();
        }

        public PanelItem(final String staticLabel, final String hint, final int length, final boolean isSeparator) {
            this();
            this.separator = isSeparator;
            this.staticLabel = staticLabel;
            this.length = length;
            final JLabel label = new JLabel(getComposedText());
            label.setToolTipText(hint);
            this.component = label;
            setLength(length);
            label.setLocation(4, 0);
            label.setFont(font);
        }

        public final Component getComponent() {
            return component;
        }

        public final int getLength() {
            return length;
        }

        public final String getStaticLabel() {
            return staticLabel;
        }

        public final void setLength(final int length) {
            component.setSize(length, 14);
            this.length = length;
        }

        public final void setStaticLabel(final String string) {
            staticLabel = string;
        }

        public final String getOptionalLabel() {
            return optionalLabel;
        }

        public final boolean isSeparator() {
            return separator;
        }

        public final void setOptionalLabel(final String optionalLabel, final String hint) {
            this.optionalLabel = optionalLabel;
            if (componentType == COMPONENT_TYPE_LABEL) {
                final JLabel label = (JLabel) component;
                label.setText(getComposedText());
                label.setToolTipText(hint);
            }
        }

        public final void setOptionalLabel(final String optionalLabel) {
            this.optionalLabel = optionalLabel;
            if (componentType == COMPONENT_TYPE_LABEL) {
                ((JLabel) component).setText(getComposedText());
                //   component.repaint();
            }
        }

        private final String getComposedText() {
            return this.getStaticLabel() + " " + this.getOptionalLabel();
        }
    }

    public final Component getManagerComponent() {
        return statusPanel;  //implement - call to super class
    }

    private void setCaretColumn(final int caretPosition) {
        int currentColumn = 0;
        if (caretPosition != 0) {
            try {
                currentColumn = caretPosition - Utilities.getRowStart(editor, caretPosition);
            } catch (BadLocationException ex) {
                Utils.processException(ex);
            }
        }
        display("column", String.valueOf(currentColumn + 1));
    }

    public final void caretUpdate(final CaretEvent e) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                setCaretColumn(e.getDot());
            }
        });
    }

    private void displayLineNumber(final String value) {
        display("line", value);
    }

    private final void displayModified(final Boolean value) {
        display("modified", (value != null && value.booleanValue()) ? AppPrefs.getLabel("statusbar.modified") : "");
    }

    public final void propertyChange(final PropertyChangeEvent evt) {
        if (evt.getPropertyName().equals(LINE_PROPERTY))
            displayLineNumber(evt.getNewValue().toString());
        else if (evt.getPropertyName().equals(MODIFIED_PROPERTY)) {
            displayModified((Boolean) evt.getNewValue());
            MainApp.getInstance().getMainAppFrame().getManagerDirector().getAreaManager().updateTabTitle();
        }
        else if (evt.getPropertyName().equals(OVERTYPE_PROPERTY))
            displayOvertype((Boolean) evt.getNewValue());
    }

    public final void specialKeyStatusChanged() {
        display("capslock", (getKeyStatus(KeyEvent.VK_CAPS_LOCK) ? AppPrefs.getLabel("statusbar.caps") : ""));
        display("numlock", (getKeyStatus(KeyEvent.VK_NUM_LOCK) ? AppPrefs.getLabel("statusbar.num") : ""));
    }

    private static boolean getKeyStatus(final int key) {
        try {
            return Toolkit.getDefaultToolkit().getLockingKeyState(key);
        } catch (UnsupportedOperationException e) { //MacOS or other systems
            return false;
        }
    }

    public final void selectedAreaChanged(final IFileInstance instance) {
        clear();
        if (instance != null && isVisible()) {
            addSection("fileName", "", "", 300);
            addSection("line", AppPrefs.getLabel("statusbar.linenumber") + " ", AppPrefs.getLabel("statusbar.hint.linePosition"), 70);
            addSection("column", AppPrefs.getLabel("statusbar.column") + " ", AppPrefs.getLabel("statusbar.hint.columnPosition"), 70);
            addSeparator("Separator1", 130);
            addSection(OVERTYPE_PROPERTY, "", AppPrefs.getLabel("statusbar.hint.overtype"), 60);
            addSection(MODIFIED_PROPERTY, "", AppPrefs.getLabel("statusbar.hint.modified"), 60);
            addSeparator("Separator2", 160);
            addSection("capslock", "", AppPrefs.getLabel("statusbar.hint.capslock"), 35);
            addSection("numlock", "", AppPrefs.getLabel("statusbar.hint.numlock"), 30);

            displayFilePath((instance.getFile() == null) ? AppPrefs.getLabel("statusbar.notsaved") : instance.getFile().getAbsolutePath());
            if (!instance.getTextComponent().equals(editor)) {
                if (editor != null) {
                    editor.removeCaretListener(this);
                    editor.removePropertyChangeListener(MODIFIED_PROPERTY, this);
                    editor.removePropertyChangeListener(LINE_PROPERTY, this);
                    editor.removePropertyChangeListener(OVERTYPE_PROPERTY, this);
                }
                editor = instance.getTextComponent();
                editor.addCaretListener(this);
                editor.addPropertyChangeListener(LINE_PROPERTY, this);
                editor.addPropertyChangeListener(MODIFIED_PROPERTY, this);
                editor.addPropertyChangeListener(OVERTYPE_PROPERTY, this);
                setCaretColumn(editor.getCaretPosition());
                displayLineNumber("1");
                displayModified((Boolean) editor.getClientProperty(MODIFIED_PROPERTY));
                displayOvertype((Boolean) editor.getClientProperty(OVERTYPE_PROPERTY));
            }
            specialKeyStatusChanged();
        } else if (editor != null) {
            editor.removeCaretListener(this);
            editor = null;
        }
    }

    public final boolean isVisible() {
        return statusPanel.isVisible();
    }

    public final void setVisible(final boolean value) {
        if (value) {
            statusPanel.setVisible(true);
            selectedAreaChanged(MainApp.getInstance().getMainAppFrame().getManagerDirector().getAreaManager().getActiveFileInstance());
        } else {
            statusPanel.setVisible(false);
            selectedAreaChanged(null);
        }
        AppPrefs.storeProperty("net.wordrider.gui.showStatusbar", value);
    }
}
