/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.dialogs;

import net.wordrider.core.AppPrefs;
import net.wordrider.utilities.Swinger;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;

/**
 * @author Vity
 */
public final class CloseDialog extends AppDialog {

    private JButton btnCancel;
    private JButton btnSave;
    private JButton selectAll;
    private JButton diselectAll;
    private JList list;

    private final static int RESULT_SAVE = 0;
    private final static int RESULT_CANCEL = 1;
    private int result = RESULT_CANCEL;
    //private Frame frame;
    private ArrayList listData;
    private boolean[] listDataChecked;

    public CloseDialog(final Frame owner, final Collection list) {
        super(owner, true);
        //this.frame = owner;
        try {
            init();
        } catch (Exception e) {
            Utils.processException(e);
        }
        prepareData(list);
        initDialogContents();
        Swinger.centerDialog(owner, this);
        this.setModal(true);
        this.setTitle(AppPrefs.getLabel("dialog.close.title"));
        this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        this.setVisible(true);
    }

    private void prepareData(final Collection list) {
        final Set set = new TreeSet(list);
        final int length = set.size();
        listDataChecked = new boolean[length];
        checkAll(true);
        listData = new ArrayList(set);
    }

    private void initDialogContents() {
        list.setModel((
                new AbstractListModel() {
                    public int getSize() {
                        return listData.size();
                    }

                    public Object getElementAt(final int i) {
                        return listData.get(i);
                    }
                }
                ));
    }

    private void btnCancel_actionPerformed() {
        cancel();
    }

    private void btnSave_actionPerformed() {
        result = RESULT_SAVE;
        cancel();
    }

    public final Collection getReturnList() {
        if (result == RESULT_CANCEL)
            return null;
        final Collection resultList = new ArrayList();
        final int length = listData.size();
        for (int i = 0; i < length; ++i) {
            if (listDataChecked[i])
                resultList.add(listData.get(i));
        }
        return resultList;
    }

    private class ActionButtonsAdapter implements ActionListener {

        public void actionPerformed(final ActionEvent e) {
            if (e.getSource().equals(btnSave))
                btnSave_actionPerformed();
            else
                btnCancel_actionPerformed();
        }
    }

    private class CheckListRenderer extends CheckRenderer {

        public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            check.setSelected(listDataChecked[index]);
            return super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);    //call to super
        }

    }


    private class MyKeyAdapter extends KeyAdapter {
        public void keyPressed(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_ENTER || e.getKeyCode() == KeyEvent.VK_SPACE) {
                toggleChecked(list.getSelectedIndex());
            }
        }
    }

    private void checkAll(final boolean value) {
        final int length = listDataChecked.length;
        for (int i = 0; i < length; ++i)
            listDataChecked[i] = value;
    }

    private class SelectAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            checkAll(e.getSource().equals(selectAll));
            list.repaint();
        }
    }

    private void toggleChecked(final int index) {
        listDataChecked[index] = !listDataChecked[index];
        final Rectangle rect = list.getCellBounds(index, index);
        list.repaint(rect);
    }

    private void init() throws Exception {

        final Container mainPanel = this.getContentPane();
        final JPanel btnPanel = new JPanel();
        mainPanel.setLayout(new BorderLayout());
        final JScrollPane scrollPane;
        mainPanel.add(scrollPane = new JScrollPane(list = new JList()), BorderLayout.CENTER);

        list.addKeyListener(new MyKeyAdapter());
        list.setCellRenderer(new CheckListRenderer());
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        scrollPane.setPreferredSize(new Dimension(220, 150));
        list.setBorder(new EmptyBorder(0, 4, 0, 0));
        list.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                toggleChecked(list.locationToIndex(e.getPoint()));
            }
        });
        mainPanel.add(btnPanel, BorderLayout.SOUTH);
        btnPanel.setLayout(new GridBagLayout());
        final Dimension buttonSize = new Dimension(80, 25);
        ActionListener selectAction = new SelectAction();
        selectAll = Swinger.getRiderButton("dialog.close.btnSelectAll");
        selectAll.addActionListener(selectAction);
        diselectAll = Swinger.getRiderButton("dialog.close.btnNone");
        diselectAll.addActionListener(selectAction);
        btnCancel = Swinger.getRiderButton("dialog.close.btnCancel");
        btnCancel.setMinimumSize(buttonSize);
        btnCancel.setPreferredSize(buttonSize);
        btnCancel.setMaximumSize(buttonSize);
        btnSave = Swinger.getRiderButton("dialog.close.btnOK");
        btnSave.setMinimumSize(buttonSize);
        btnSave.setPreferredSize(buttonSize);
        btnSave.setMaximumSize(buttonSize);

        final ActionListener actionButtonListener = new ActionButtonsAdapter();
        btnSave.addActionListener(actionButtonListener);
        btnCancel.addActionListener(actionButtonListener);
        btnPanel.add(selectAll, new GridBagConstraints(0, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 4, 4, 5), 0, 0));
        btnPanel.add(diselectAll, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(4, 8, 4, 5), 0, 0));
        btnPanel.add(btnSave, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 23, 4, 5), 0, 0));
        btnPanel.add(btnCancel, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0
                , GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets(4, 8, 4, 5), 0, 0));
        this.setSize(350, 195);
        this.pack();
    }

}
