/*
* Copyright (C) 2004 Ladislav Vitasek
* info@wordrider.net
* http://www.wordrider.net
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package net.wordrider.gui;

import net.wordrider.core.AppPrefs;
import net.wordrider.core.MainApp;
import net.wordrider.utilities.Consts;
import net.wordrider.utilities.Utils;

import javax.swing.*;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;
import java.io.File;
import java.io.FilenameFilter;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * @author Vity
 */
public class LookAndFeels {

    private static final String KUNSTSTOFF = "com.incors.plaf.kunststoff.KunststoffLookAndFeel";

    private static final String lookAndFieldSelectedKey = "lookAndFeel";
    private static final String themeSelectedKey = "theme";
    private static String selectedLookAndFeel = AppPrefs.getProperty(lookAndFieldSelectedKey, KUNSTSTOFF);
    private static String selectedTheme = AppPrefs.getProperty(themeSelectedKey, null);

    private static final ClassLoader classLoader = initClassLoader();

    static {
        if (selectedTheme == null && selectedLookAndFeel.equals(KUNSTSTOFF))
            selectedTheme = WordRiderMetalTheme.class.getName();
    }

    private static ClassLoader initClassLoader() {
        String path = AppPrefs.getAppPath() + "lib";
        Utils.log(Utils.DEBUG_LOG, "Loading plugin path " + path);
        final File file = new File(path);
        ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        try {
            if (file.exists()) {
                File[] jars = file.listFiles(new FilenameFilter() {
                    public boolean accept(File dir, String name) {
                        return name.endsWith(".jar");
                    }
                });
                final int jarsCount = jars.length;
                URL[] urls = new URL[jarsCount];
                final boolean isDebug = Utils.isDefinedForLog(Utils.DEBUG_LOG);
                for (int i = 0; i < jarsCount; ++i) {
                    urls[i] = jars[i].toURL();
                    if (isDebug)
                        Utils.logDebug("Loading URL with jar " + urls[i]);
                }
                classLoader = new URLClassLoader(urls, classLoader);
            }
            return classLoader;
        } catch (Exception e) {
            Utils.processException(e);
            return classLoader;
        }
    }

    public static final boolean isSelectedLaF(final String className, final String themeClassName) {
        if (className.equals(selectedLookAndFeel)) {
            if (selectedTheme != null)
                return selectedTheme.equals(themeClassName);
            else
                return (themeClassName == null);
        }
        return false;
    }

    public static final void storeSelectedLaF(final String lookAndFeelClassName, final String themeClassName) {
        AppPrefs.storeProperty(lookAndFieldSelectedKey, selectedLookAndFeel = lookAndFeelClassName);
        if (themeClassName == null) {
            selectedTheme = null;
            AppPrefs.removeProperty(themeSelectedKey);
        } else
            AppPrefs.storeProperty(themeSelectedKey, selectedTheme = themeClassName);
    }

    public static final void loadLookAndFeelSettings() {
        loadLookAndFeel(selectedLookAndFeel, selectedTheme, false);
    }

    public static final Class isPresent(final String className) {
        try {
            return classLoader.loadClass(className);
        } catch (ClassNotFoundException e) {
            if (Utils.isDefinedForLog(Utils.DEBUG_LOG)) {
                Utils.logDebug("Possible problem.Look and feel class/theme " + className + " was not detected(not in /lib?).Check " + Consts.LAFSDIRFILE);
            }
            return null;
        }
    }

    public static final boolean loadLookAndFeel(final String lookAndFeelClassName, final String themeClassName, boolean updateTree) {
        try {
            Thread.currentThread().setContextClassLoader(classLoader);
            if (lookAndFeelClassName.equals(KUNSTSTOFF))
                initKunststoff();
            else
                initLaf(lookAndFeelClassName, themeClassName);
        } catch (Exception e) {
            Utils.log(Utils.ERROR_LOG, lookAndFeelClassName + " was not found.");
            if (Utils.isDefinedForLog(Utils.DEBUG_LOG))
                Utils.processException(e);
            if (!lookAndFeelClassName.equals(KUNSTSTOFF)) {
                try {
                    initKunststoff();
                    initKunststoff();
                } catch (Exception ex) {
                    Utils.log(Utils.ERROR_LOG, "Kunststoff Look and Feel was not found. Using a default metal theme.");
                    return false;
                }
            }
            return false;
        } finally {
            if (updateTree) {
                SwingUtilities.updateComponentTreeUI(MainApp.getInstance().getMainAppFrame());
                UIManager.getLookAndFeel().getDefaults();
                //MainApp.getInstance().getMainAppFrame().pack();
                MainApp.getInstance().getMainAppFrame().invalidate();
                MainApp.getInstance().getMainAppFrame().validate();
                MainApp.getInstance().getMainAppFrame().repaint();
                //  UIManager.getLookAndFeel().getDefaults();
            }
        }
        return true;
    }

    private static void initKunststoff() throws Exception {
        initLafWithTheme(KUNSTSTOFF, new WordRiderMetalTheme());
    }

    private static void initLafWithTheme(final String lookAndFeelClassName, final MetalTheme metalTheme) throws Exception {
        final LookAndFeel laf = (LookAndFeel) classLoader.loadClass(lookAndFeelClassName).newInstance();
//Object[] keys = UIManager.getDefaults().keySet().toArray();
//for (int i=0 ; i < keys.length ; i++) {
//  UIManager.put(keys[i],null);
//}

        if (metalTheme != null && laf instanceof MetalLookAndFeel) {
            laf.getClass().getMethod("setCurrentTheme", new Class[]{MetalTheme.class}).invoke(laf, new Object[]{metalTheme});
        }
        UIManager.put("ClassLoader", classLoader);
        UIManager.setLookAndFeel(laf);
        UIManager.put("ClassLoader", classLoader);
    }

    private static void initLaf(final String lookAndFeelClassName, final String themeClassName) throws Exception {
        if (themeClassName != null) {
            final Class themeClass = isPresent(themeClassName);
            if (themeClass != null) {
                Object instanceTheme = themeClass.newInstance();
                if (instanceTheme instanceof MetalTheme)
                    initLafWithTheme(lookAndFeelClassName, (MetalTheme) instanceTheme);
                else {
                    Utils.log(Utils.WARN_LOG, "Theme " + instanceTheme.getClass().getName() + " cannot be set.Theme is not an instance of Metaltheme");
                    initLafWithTheme(lookAndFeelClassName, null);
                }
            }
        } else
            initLafWithTheme(lookAndFeelClassName, null);
    }
}
